from __future__ import print_function, absolute_import, division

import logging
import re
from tempfile import NamedTemporaryFile

import boto3
from cfn_sphere import StackActionHandler
from cfn_sphere.stack_configuration import Config

from .stack_templates import RECORDSET_STACK, SES_EMAIL_BUCKET_STACK

LOGGER = logging.getLogger("aws-certificate-management")
DNS_STACK_NAME_POSTFIX = "-ses-dns-records"
BUCKET_STACK_NAME_POSTFIX = "-email-bucket"
REGION = "eu-west-1"

# These files will be closed (and deleted) when Python terminates.
recordset_template = NamedTemporaryFile(suffix=".json")
ses_template = NamedTemporaryFile(suffix=".json")
recordset_template.write(RECORDSET_STACK)
recordset_template.flush()
ses_template.write(SES_EMAIL_BUCKET_STACK)
ses_template.flush()


def prepare_domain(domain):
    # Stack names in CFN-sphere may not contain anything but letters,
    # numbers and "-".
    return re.sub("[^-a-zA-Z0-9]", "", domain)


def get_dns_stack_name(domain):
    return prepare_domain(domain) + DNS_STACK_NAME_POSTFIX


def get_bucket_stack_name(domain):
    return prepare_domain(domain) + BUCKET_STACK_NAME_POSTFIX


def get_stack_action_handler(domain, verification_token=None, dkim_tokens=None):
    verification_token = verification_token or ""
    dkim_tokens = dkim_tokens or ["", "", ""]

    return StackActionHandler(config=Config(config_dict={
        'region': REGION,
        'stacks': {
            get_dns_stack_name(domain): {
                'template-url': recordset_template.name,
                'parameters': {
                    'dnsBaseName': domain + ".",
                    'dkimOne': dkim_tokens[0],
                    'dkimTwo': dkim_tokens[1],
                    'dkimThree': dkim_tokens[2],
                    'verifyTxt': verification_token
                }
            },
            get_bucket_stack_name(domain): {
                'template-url': ses_template.name,
            }
        }
    }))


def normalize_domain(domain):
    if domain.startswith("*."):
        return domain[2:]
    elif domain.startswith("www."):
        return domain[4:]
    return domain


def create_ses_dns_records(domain):
    LOGGER.info("Creating DNS records to configure e-mail for your domain")
    domain = normalize_domain(domain)
    ses = boto3.client('ses', region_name=REGION)
    domain_identity = ses.verify_domain_identity(Domain=domain)
    verification_token = domain_identity['VerificationToken']
    dkim_tokens = ses.verify_domain_dkim(Domain=domain)['DkimTokens']

    stack_handler = get_stack_action_handler(domain, verification_token, dkim_tokens)
    LOGGER.info("Creating CFN stacks %s and %s",
                get_dns_stack_name(domain), get_bucket_stack_name(domain))
    stack_handler.create_or_update_stacks()

    stack_outputs = stack_handler.cfn.get_stack_outputs()
    s3_bucket_for_mail = stack_outputs[get_bucket_stack_name(domain)]['bucketName']
    LOGGER.info("Mail for postmaster@<yourdomain> will be delivered to "
                "S3 bucket %s", s3_bucket_for_mail)
    return s3_bucket_for_mail


def delete_items_in_bucket(s3_bucket):
    s3_client = boto3.client('s3')
    for item in s3_client.list_objects(Bucket=s3_bucket).get('Contents', []):
        s3_client.delete_object(Key=item['Key'], Bucket=s3_bucket)


def delete_ses_dns_records_and_bucket(domain):
    LOGGER.info("Deleting DNS records that configure e-mail for your domain")
    domain = normalize_domain(domain)
    stack_handler = get_stack_action_handler(domain)

    stacks_dict = stack_handler.cfn.get_stack_outputs()
    bucket_stack_outputs = stacks_dict[get_bucket_stack_name(domain)]
    bucket_name = bucket_stack_outputs['bucketName']
    LOGGER.info("Deleting all items in S3 bucket %r to prepare stack deletion",
                bucket_name)
    delete_items_in_bucket(bucket_name)

    stack_handler.delete_stacks()
    LOGGER.info("Deletion of DNS records and mail bucket complete")



RECORDSET_STACK = br"""
{
  "Description": "Route53 record sets to use SES to receive mail for a domain",
  "Parameters": {
    "dnsBaseName": {
      "Type": "String",
      "Description": "The name of the domain. You must specify a fully qualified domain name that ends with a period as the last label indication. If you omit the final period, AWS CloudFormation adds it."
    },
    "dkimOne": {
      "Type": "String",
      "Description": "sub domain value for '.dkim.amazonses.com.' CNAME record"
    },
    "dkimTwo": {
      "Type": "String",
      "Description": "sub domain value for '.dkim.amazonses.com.' CNAME record"
    },
    "dkimThree": {
      "Type": "String",
      "Description": "sub domain value for '.dkim.amazonses.com.' CNAME record"
    },
    "verifyTxt": {
      "Type": "String",
      "Description": "verify domain TXT record value"
    }
  },
  "AWSTemplateFormatVersion": "2010-09-09",
  "Resources": {
    "mxDnsRecord": {
      "Type": "AWS::Route53::RecordSet",
      "Properties": {
        "HostedZoneName": {
          "Ref": "dnsBaseName"
        },
        "TTL": "900",
        "Type": "MX",
        "Name": {
          "Ref": "dnsBaseName"
        },
        "ResourceRecords": [
          "10 inbound-smtp.eu-west-1.amazonaws.com."
        ]
      }
    },
    "dkimDnsRecordThree": {
      "Type": "AWS::Route53::RecordSet",
      "Properties": {
        "HostedZoneName": {
          "Ref": "dnsBaseName"
        },
        "TTL": "900",
        "Type": "CNAME",
        "Name": {
          "Fn::Join": [
            ".",
            [
              {
                "Ref": "dkimThree"
              },
              "_domainkey",
              {
                "Ref": "dnsBaseName"
              }
            ]
          ]
        },
        "ResourceRecords": [
          {
            "Fn::Join": [
              "",
              [
                {
                  "Ref": "dkimThree"
                },
                ".dkim.amazonses.com."
              ]
            ]
          }
        ]
      }
    },
    "dkimDnsRecordOne": {
      "Type": "AWS::Route53::RecordSet",
      "Properties": {
        "HostedZoneName": {
          "Ref": "dnsBaseName"
        },
        "TTL": "900",
        "Type": "CNAME",
        "Name": {
          "Fn::Join": [
            ".",
            [
              {
                "Ref": "dkimOne"
              },
              "_domainkey",
              {
                "Ref": "dnsBaseName"
              }
            ]
          ]
        },
        "ResourceRecords": [
          {
            "Fn::Join": [
              "",
              [
                {
                  "Ref": "dkimOne"
                },
                ".dkim.amazonses.com."
              ]
            ]
          }
        ]
      }
    },
    "verifyDomainDnsRecord": {
      "Type": "AWS::Route53::RecordSet",
      "Properties": {
        "HostedZoneName": {
          "Ref": "dnsBaseName"
        },
        "TTL": "900",
        "Type": "TXT",
        "Name": {
          "Fn::Join": [
            ".",
            [
              "_amazonses",
              {
                "Ref": "dnsBaseName"
              }
            ]
          ]
        },
        "ResourceRecords": [
          {
            "Fn::Join": [
              "",
              [
                "\"",
                {
                  "Ref": "verifyTxt"
                },
                "\""
              ]
            ]
          }
        ]
      }
    },
    "autodiscoverDnsRecord": {
      "Type": "AWS::Route53::RecordSet",
      "Properties": {
        "HostedZoneName": {
          "Ref": "dnsBaseName"
        },
        "TTL": "900",
        "Type": "CNAME",
        "Name": {
          "Fn::Join": [
            ".",
            [
              "autodiscover",
              {
                "Ref": "dnsBaseName"
              }
            ]
          ]
        },
        "ResourceRecords": [
          "autodiscover.mail.eu-west-1.awsapps.com."
        ]
      }
    },
    "dkimDnsRecordTwo": {
      "Type": "AWS::Route53::RecordSet",
      "Properties": {
        "HostedZoneName": {
          "Ref": "dnsBaseName"
        },
        "TTL": "900",
        "Type": "CNAME",
        "Name": {
          "Fn::Join": [
            ".",
            [
              {
                "Ref": "dkimTwo"
              },
              "_domainkey",
              {
                "Ref": "dnsBaseName"
              }
            ]
          ]
        },
        "ResourceRecords": [
          {
            "Fn::Join": [
              "",
              [
                {
                  "Ref": "dkimTwo"
                },
                ".dkim.amazonses.com."
              ]
            ]
          }
        ]
      }
    }
  }
}
"""

SES_EMAIL_BUCKET_STACK = b"""
{
  "AWSTemplateFormatVersion": "2010-09-09",
  "Description": "Destination bucket for emails received by SES",
  "Resources": {
    "bucket": {
      "Type": "AWS::S3::Bucket",
      "Properties": {
        "LifecycleConfiguration": {
          "Rules": [
            {
              "Status": "Enabled",
              "ExpirationInDays": 14
            }
          ]
        }
      }
    },
    "bucketPolicy": {
      "Type": "AWS::S3::BucketPolicy",
      "Properties": {
        "Bucket": {
          "Ref": "bucket"
        },
        "PolicyDocument": {
          "Statement": {
            "Sid": "GiveSESPermissionToWriteEmail",
            "Effect": "Allow",
            "Principal": {
              "Service": [
                "ses.amazonaws.com"
              ]
            },
            "Action": [
              "s3:PutObject"
            ],
            "Resource": {
              "Fn::Join": [
                "",
                [
                  "arn:aws:s3:::",
                  {
                    "Ref": "bucket"
                  },
                  "/*"
                ]
              ]
            },
            "Condition": {
              "StringEquals": {
                "aws:Referer": {
                  "Ref": "AWS::AccountId"
                }
              }
            }
          }
        }
      }
    }
  },
  "Outputs": {
    "bucketName": {
      "Value": { "Ref": "bucket" },
      "Description": "Name of the bucket where SES delivers mail to"
    }
  }
}
"""



from __future__ import print_function, absolute_import, division

import json
import logging
import subprocess
import time

from .configure_dns import normalize_domain

LOGGER = logging.getLogger("aws-certificate-management")
REGION = "--region=eu-west-1"


def run(command):
    # On clean build systems, the "aws ...." commands fail because no default
    # region is configured. We need to explicitly name it in every command.
    command.append(REGION)

    LOGGER.debug("Running this command: %r", command)
    subprocess.check_call(command)


def get_active_rule_set():
    """Return the name of the active rule set (if any), else None"""
    # Avoid throttling
    time.sleep(1)
    currently_active_rule_set = subprocess.check_output([
        'aws ses describe-active-receipt-rule-set ' + REGION],
        shell=True)
    if not currently_active_rule_set:
        return
    return json.loads(currently_active_rule_set)['Metadata']['Name']


def deactivate_rule_set_if_active(rule_set_name):
    if get_active_rule_set() != rule_set_name:
        return

    # Without parameter, set-active-receipt-rule-set deactivates the
    # currently active rule.
    run(['aws', 'ses', 'set-active-receipt-rule-set'])


def delete_rule_set(rule_set_name):
    """Delete the given rule set if it exists

    If the rule set is currently active (which would normally prevent
    deletion), the rule set is deactivated first.
    """
    deactivate_rule_set_if_active(rule_set_name)
    run(['aws', 'ses', 'delete-receipt-rule-set',
         '--rule-set-name', rule_set_name])


def generate_rule(domain, s3_bucket):
    rule = {
        "Name": "postmaster",
        "Enabled": True,
        "Recipients": ["postmaster@{0}".format(domain)],
        "Actions": [{
            "S3Action": {
                "BucketName": s3_bucket
            }
        }]
    }
    return json.dumps(rule)


def create_rule_set(rule_set_name, rule):
    """Create the given rule set and activate it

    This assumes that no rule set of that name currently exists
    """
    run(['aws', 'ses', 'create-receipt-rule-set',
         '--rule-set-name', rule_set_name])

    run(['aws', 'ses', 'create-receipt-rule',
         '--rule-set-name', rule_set_name, '--rule', rule])

    run(['aws', 'ses', 'set-active-receipt-rule-set',
         '--rule-set-name', rule_set_name])


def get_rule_set_name(domain):
    normalized_domain = normalize_domain(domain)
    return "standard_addresses_for_{0}".format(normalized_domain)


def setup_ses_rule_set(domain, s3_bucket):
    normalized_domain = normalize_domain(domain)
    rule = generate_rule(normalized_domain, s3_bucket)
    rule_set_name = get_rule_set_name(domain)

    delete_rule_set(rule_set_name)
    create_rule_set(rule_set_name, rule)


def cleanup_ses_rule_set(domain):
    rule_set_name = get_rule_set_name(domain)
    delete_rule_set(rule_set_name)
    LOGGER.info("Deletion of SES mail rule finished")
    LOGGER.warn("Check if you need to set an active rule in SES!")



from __future__ import print_function, absolute_import, division

import boto3
import logging

from .ses import setup_ses_rule_set, cleanup_ses_rule_set
from .configure_dns import create_ses_dns_records
from .configure_dns import delete_ses_dns_records_and_bucket


__all__ = ["setup_ses_rule_set", "create_ses_dns_records",
           "delete_ses_dns_records_and_bucket", "cleanup_ses_rule_set",
           "setup_certificate", "cleanup"]


def setup_certificate(domain, region):
    mail_bucket_name = create_ses_dns_records(domain)
    setup_ses_rule_set(domain, mail_bucket_name)

    client = boto3.client('acm', region_name=region)
    response = client.request_certificate(DomainName=domain)
    certificate_arn = response['CertificateArn']
    logging.getLogger('aws-certificate-management').info(
        "Your certificate ARN is %r", certificate_arn)


def cleanup(domain):
    delete_ses_dns_records_and_bucket(domain)
    cleanup_ses_rule_set(domain)



#!/usr/bin/env python

from setuptools import setup
from setuptools.command.install import install as _install

class install(_install):
    def pre_install_script(self):
        pass

    def post_install_script(self):
        pass

    def run(self):
        self.pre_install_script()

        _install.run(self)

        self.post_install_script()

if __name__ == '__main__':
    setup(
        name = 'aws-certificate-management',
        version = '67',
        description = '''''',
        long_description = '''Tool to automate certificate creation in AWS''',
        author = "",
        author_email = "",
        license = '',
        url = 'https://github.com/ImmobilienScout24/aws-certificate-management',
        scripts = ['scripts/aws-certificate-management'],
        packages = ['aws_certificate_management'],
        py_modules = [],
        classifiers = [
            'Development Status :: 3 - Alpha',
            'Programming Language :: Python'
        ],
        entry_points = {},
        data_files = [],
        package_data = {},
        install_requires = [
            'awscli',
            'boto3',
            'cfn-sphere'
        ],
        dependency_links = [],
        zip_safe=True,
        cmdclass={'install': install},
    )
